import get from 'lodash.get';
import { subYears, addYears, parse, isValid, isBefore, isAfter, isEqual } from 'date-fns';
import { buildYup } from 'json-schema-to-yup';
import isArray from 'lodash.isarray';

function _extends() {
  _extends = Object.assign ? Object.assign.bind() : function (target) {
    for (var i = 1; i < arguments.length; i++) {
      var source = arguments[i];
      for (var key in source) {
        if (Object.prototype.hasOwnProperty.call(source, key)) {
          target[key] = source[key];
        }
      }
    }
    return target;
  };
  return _extends.apply(this, arguments);
}
function _objectWithoutPropertiesLoose(source, excluded) {
  if (source == null) return {};
  var target = {};
  var sourceKeys = Object.keys(source);
  var key, i;
  for (i = 0; i < sourceKeys.length; i++) {
    key = sourceKeys[i];
    if (excluded.indexOf(key) >= 0) continue;
    target[key] = source[key];
  }
  return target;
}

// Desc: Configs for the translation that is used in the validation messages
const translatesConfig = {
  nullable: 'generic-form:validation.nullable',
  required: 'generic-form:validation.required',
  min: 'generic-form:validation.min',
  max: 'generic-form:validation.max',
  maxLength: 'generic-form:validation.max_length',
  minLength: 'generic-form:validation.min_length',
  pattern: 'generic-form:validation.pattern',
  format: 'generic-form:validation.format',
  email: 'generic-form:validation.email',
  minDate: 'generic-form:validation.min_date',
  maxDate: 'generic-form:validation.max_date',
  wrongRange: 'generic-form:validation.wrong_range',
  wrongDate: 'generic-form:validation.wrong_range',
  trim: 'generic-form:validation.trim',
  oneOf: 'generic-form:validation.one_of'
};
// Desc: Enum for the validation error types
var EValidationTypes;
(function (EValidationTypes) {
  EValidationTypes["nullable"] = "nullable";
  EValidationTypes["required"] = "required";
  EValidationTypes["min"] = "min";
  EValidationTypes["max"] = "max";
  EValidationTypes["oneOf"] = "oneOf";
  EValidationTypes["maxLength"] = "maxLength";
  EValidationTypes["minLength"] = "minLength";
  EValidationTypes["pattern"] = "pattern";
  EValidationTypes["format"] = "format";
  EValidationTypes["minDate"] = "minDate";
  EValidationTypes["maxDate"] = "maxDate";
  EValidationTypes["wrongRange"] = "wrongRange";
  EValidationTypes["wrongDate"] = "wrongDate";
  EValidationTypes["trim"] = "trim";
})(EValidationTypes || (EValidationTypes = {}));

var EFieldTypes;
(function (EFieldTypes) {
  EFieldTypes["TEXT"] = "text";
  EFieldTypes["NUMBER"] = "number";
  EFieldTypes["SWITCH"] = "switch";
  EFieldTypes["DATE"] = "date";
  EFieldTypes["DATE_RANGE"] = "dateRange";
  EFieldTypes["AUTOCOMPLETE"] = "autoComplete";
  EFieldTypes["FILE"] = "file";
  EFieldTypes["ARRAY"] = "array";
  EFieldTypes["MOBILE_PHONE"] = "mobilePhone";
  EFieldTypes["MULTIPLE_AUTOCOMPLETE"] = "autoCompleteMultiple";
  EFieldTypes["RADIOBUTTON"] = "radioButton";
  EFieldTypes["EMPTY"] = "empty";
  EFieldTypes["CHECKBOX"] = "checkbox";
  EFieldTypes["RADIOBUTTON_MATRIX"] = "radioButtonMatrix";
  EFieldTypes["RATING"] = "rating";
})(EFieldTypes || (EFieldTypes = {}));

/**
 * @param formDataChildren - array of form fields
 * @param acc - accumulator, used for recursion
 * @returns {[key: string]: any} - object with the initial values for the form
 * @description getInitialValues function returns an object with the initial values for the form
 * based on the initial values provided in the json schema
 */
const getInitialValues = (formDataChildren, acc = {}) => {
  return formDataChildren.reduce((acc, item) => {
    if (isFormGroup(item)) {
      return getInitialValues(item.children, acc);
    }
    acc[item.fieldName] = item.initialValue === null ? undefined : item.initialValue;
    return acc;
  }, acc);
};
/**
 * @param schema - yup schema
 * @param fieldName - field name to get the rule value from
 * @param ruleName - rule name to get the value from
 * @returns {any} - rule value
 * @description getRuleValueFromSchema function returns the value of a rule from a yup schema
 * based on the field name and rule name. This value is needed for passing meta to translations.
 * @example getRuleValueFromSchema(schema, 'email', 'minLength') => 6
 */
const getRuleValueFromSchema = (schema, fieldName, ruleName) => {
  const describedSchema = schema.describe();
  const rules = get(describedSchema, `fields.${fieldName}.tests`, []);
  const rule = rules.find(rule => rule.name === ruleName);
  return get(rule, `params.${ruleName}`, undefined);
};
/**
 * @param type - validation type
 * @param value - value to be validated
 * @param t - translation function(optional)
 * @param validationParam - validation param(meta) to be passed to the translation function(optional) or error object
 * @returns {any} - error object or function, or translated string
 * @description getFieldError function returns an error object or function, or translated string based on the validation type
 */
const getFieldError = (type, value, t, validationParam) => {
  let translationKey = get(translatesConfig, `${type}`);
  switch (type) {
    case EValidationTypes.nullable:
      return t ? t(translationKey) : createErrorObject(translationKey);
    case EValidationTypes.required:
      return t ? t(translationKey) : createErrorObject(translationKey);
    case EValidationTypes.min:
      return t ? ({
        min
      }) => t(translationKey, {
        param: min
      }) : createErrorObject(translationKey, {
        param: validationParam
      });
    case EValidationTypes.max:
      return t ? ({
        max
      }) => t(translationKey, {
        param: max
      }) : createErrorObject(translationKey, {
        param: validationParam
      });
    case EValidationTypes.maxLength:
      return t ? ({
        maxLength
      }) => t(translationKey, {
        param: maxLength
      }) : createErrorObject(translationKey, {
        param: validationParam
      });
    case EValidationTypes.minLength:
      return t ? ({
        minLength
      }) => t(translationKey, {
        param: minLength
      }) : createErrorObject(translationKey, {
        param: validationParam
      });
    case EValidationTypes.format:
      if (value) {
        translationKey = get(translatesConfig, `${type}`) ? get(translatesConfig, `${type}`) : translationKey;
      }
      return t ? t(translationKey) : createErrorObject(translationKey, {
        param: validationParam
      });
    case EValidationTypes.trim:
    case EValidationTypes.oneOf:
    case EValidationTypes.pattern:
      return t ? t(translationKey) : createErrorObject(translationKey, {
        param: validationParam
      });
    default:
      return null;
  }
};
/**
 * @param validationObject
 * @param formValues - form values to be validated
 * @returns {IValidationErrors} - object with the validation errors
 * @description makeCrossFieldValidation function makes custom cross-field validation
 * and returns an object with the validation errors
 */
const makeCrossFieldValidation = (validationObject, formValues) => {
  const errors = {};
  try {
    const keys = Object.keys(validationObject);
    keys.forEach(fieldName => {
      const rules = validationObject[fieldName];
      const fieldErrors = {
        property: fieldName,
        constraints: {}
      };
      rules == null || rules.forEach((rule, id) => {
        const {
          errorKey,
          testCondition,
          meta
        } = rule;
        const testFunction = functionFromString(testCondition);
        let shouldIncludeError = false;
        try {
          shouldIncludeError = testFunction(formValues);
        } catch (e) {
          console.log('Error during evaluating custom validation rule. This issue wont cause an error, current validation was skipped', e);
        }
        if (shouldIncludeError) {
          fieldErrors.constraints[id] = createErrorObject(errorKey, meta);
        }
      });
      if (Object.keys(fieldErrors.constraints).length > 0) {
        errors[fieldName] = fieldErrors;
      }
    });
    return errors;
  } catch (e) {
    console.warn('Error in makeCrossFieldValidation: ', e);
    return {};
  }
};
/**
 * @param fieldName - field name
 * @param values - form values
 * @param errors - object with the validation errors
 * @returns {IValidationErrors} - object with the validation errors
 * @description validateDateRangeField function validates date range field
 * and returns an object with the validation errors
 */
const validateDateRangeField = (fieldName, values, errors = {}) => {
  const minDate = subYears(new Date(), 100);
  const maxDate = addYears(new Date(), 100);
  const error = {
    property: fieldName,
    constraints: {}
  };
  const dateRangeUnmapped = get(values, fieldName, [null, null]);
  if (!isArray(dateRangeUnmapped)) {
    error.constraints[EValidationTypes.wrongDate] = {
      translationKey: translatesConfig[EValidationTypes.wrongDate],
      fallbackLanguageTranslation: 'This field expected to be array of strings',
      meta: {}
    };
    if (!errors[fieldName]) {
      errors[fieldName] = error;
    } else {
      errors[fieldName].constraints = _extends({}, errors[fieldName].constraints, error.constraints);
    }
    return errors;
  }
  const dateRange = dateRangeUnmapped.map(value => value ? parse(value, 'yyyy-MM-dd', new Date()) : null);
  if (!isValid(dateRange[0]) || !isValid(dateRange[1])) {
    error.constraints[EValidationTypes.wrongDate] = {
      translationKey: translatesConfig[EValidationTypes.wrongDate],
      fallbackLanguageTranslation: 'Some date in range is invalid',
      meta: {}
    };
  }
  if (!dateRange[0] || !dateRange[1]) {
    error.constraints[EValidationTypes.required] = {
      translationKey: translatesConfig[EValidationTypes.required],
      fallbackLanguageTranslation: 'Some date in range is empty',
      meta: {}
    };
  }
  if (dateRange[0] && isBefore(dateRange[0], minDate) || dateRange[1] && isBefore(dateRange[1], minDate)) {
    error.constraints[EValidationTypes.minDate] = {
      translationKey: translatesConfig[EValidationTypes.minDate],
      fallbackLanguageTranslation: 'Date range should be between 100 years ago and 100 years in the future',
      meta: {}
    };
  }
  if (dateRange[1] && isAfter(dateRange[1], maxDate) || dateRange[0] && isAfter(dateRange[0], maxDate)) {
    error.constraints[EValidationTypes.maxDate] = {
      translationKey: translatesConfig[EValidationTypes.maxDate],
      fallbackLanguageTranslation: 'Date range should be between 100 years ago and 100 years in the future',
      meta: {}
    };
  }
  if (dateRange[0] && dateRange[1] && (isAfter(dateRange[0], dateRange[1]) || isEqual(dateRange[0], dateRange[1]))) {
    error.constraints[EValidationTypes.wrongRange] = {
      translationKey: translatesConfig[EValidationTypes.wrongRange],
      fallbackLanguageTranslation: 'Start date should be before end date',
      meta: {}
    };
  }
  if (Object.keys(error.constraints).length > 0) {
    if (!errors[fieldName]) {
      errors[fieldName] = error;
    } else {
      errors[fieldName].constraints = _extends({}, errors[fieldName].constraints, error.constraints);
    }
  }
  return errors;
};
/**
 * @param jsonSchema - json validation schema
 * @param t - translation function(optional)
 * @returns {AnyObjectSchema} - yup validation schema
 * @description buildYupSchema function builds a yup validation schema based on the json validation schema
 */
const buildYupSchema = (jsonSchema, t) => {
  return buildYup(jsonSchema, t ? {
    errMessages: getErrorMessages(jsonSchema, t)
  } : undefined);
};
/**
 * @param form {IForm} - form json schema
 * @param valuesToValidate {any} - form values to be validated
 * @param skipCrossField {boolean} - skip cross-field validation(optional)
 * @returns {Promise<unknown>} - object with the validation errors or error that has occurred during the validation
 * @description validateValues function validates the form values based on the json form object
 */
const validateValues = async ({
  children,
  jsonSchema,
  formSettings
}, valuesToValidate, skipCrossField = false) => {
  let errors = {};
  const flattenedValues = flattenValues(valuesToValidate);
  const trimmedValues = trimValuesInObject(flattenedValues);
  try {
    const validationSchema = buildYupSchema(jsonSchema);
    const dateRangeFields = getFieldNamesByType(children, EFieldTypes.DATE_RANGE);
    if (!skipCrossField) {
      const formFieldNames = Object.keys(formSettings);
      const crossFieldValidations = formFieldNames.reduce((acc, el) => {
        if (!formSettings[el].crossFieldValidations) {
          return acc;
        }
        acc[el] = formSettings[el].crossFieldValidations;
        return acc;
      }, {});
      errors = makeCrossFieldValidation(crossFieldValidations, trimmedValues);
    }
    if (dateRangeFields.length > 0) {
      dateRangeFields.forEach(fieldName => {
        errors = validateDateRangeField(fieldName, trimmedValues, errors);
      });
    }
    await validationSchema.validate(trimmedValues, {
      abortEarly: false
    });
  } catch (e) {
    if (get(e, 'name') === 'ValidationError') {
      errors = formatYupValidationErrors(e, errors);
    } else {
      throw e;
    }
  }
  return {
    errors: Object.values(errors)
  };
};

const _excluded = ["grid", "renderCondition", "disabledCondition"],
  _excluded2 = ["crossFieldValidations"];
const buildOptionsFromChildren = (children, formOptions = {}) => {
  return children.reduce((acc, child) => {
    if (isFormGroup(child)) {
      return buildOptionsFromChildren(child.children, acc);
    }
    const fieldLayout = child.grid || {
      md: 12,
      sm: 12,
      xs: 12
    };
    acc[child.fieldName] = {
      crossFieldConditions: {
        renderCondition: child.renderCondition,
        disabledCondition: child.disabledCondition
      },
      fieldLayout
    };
    return acc;
  }, formOptions);
};
const sanitizeChildrenFunction = children => {
  return children.map(child => {
    if (isFormGroup(child)) {
      return _extends({}, child, {
        children: sanitizeChildrenFunction(child.children)
      });
    }
    const sanitizedChild = _objectWithoutPropertiesLoose(child, _excluded);
    return sanitizedChild;
  });
};
const parseFormV1 = form => {
  const {
      submitButtonText,
      cancelButtonText,
      validationObject: {
        crossFieldValidations
      },
      children
    } = form,
    sanitizedValidationObject = _objectWithoutPropertiesLoose(form.validationObject, _excluded2);
  const formOptions = buildOptionsFromChildren(children);
  const fieldsWithCrossFieldValidations = Object.keys(crossFieldValidations);
  fieldsWithCrossFieldValidations.forEach(key => {
    formOptions[key].crossFieldValidations = crossFieldValidations[key];
  });
  const sanitizedChildren = sanitizeChildrenFunction(children);
  return {
    formatVersion: 'parsed',
    jsonSchema: sanitizedValidationObject,
    formSettings: formOptions,
    children: sanitizedChildren,
    submitButtonText,
    cancelButtonText
  };
};

const parseFormV2 = form => {
  const {
    submitButtonText,
    cancelButtonText,
    validationObject,
    crossFieldValidations,
    crossFieldConditions,
    formLayout,
    children
  } = form;
  const allFieldNames = Object.keys(validationObject.properties);
  const formOptions = allFieldNames.reduce((acc, fieldName) => {
    const fieldLayout = formLayout[fieldName] || {
      md: 12,
      sm: 12,
      xs: 12
    };
    acc[fieldName] = {
      crossFieldValidations: crossFieldValidations == null ? void 0 : crossFieldValidations[fieldName],
      crossFieldConditions: crossFieldConditions == null ? void 0 : crossFieldConditions[fieldName],
      fieldLayout
    };
    return acc;
  }, {});
  return {
    formatVersion: 'parsed',
    jsonSchema: validationObject,
    formSettings: formOptions,
    children,
    submitButtonText,
    cancelButtonText
  };
};

/**
 * @param jsonSchema - jsonSchema validation object
 * @param t - i18n translate function
 * @returns {any}
 * @description getErrorMessages function returns an object with the error messages for each field in the form based on the jsonSchema
 */
const getErrorMessages = (jsonSchema, t) => {
  const errMessages = {};
  Object.keys(jsonSchema.properties).forEach(property => {
    errMessages[property] = {};
    Object.keys(jsonSchema.properties[property]).forEach(key => {
      const value = jsonSchema.properties[property][key];
      errMessages[property][key] = getFieldError(key, value, t);
      if (Object.keys(jsonSchema).includes(key)) {
        errMessages[property].required = getFieldError('required', null, t);
      }
    });
  });
  return errMessages;
};
/**
 * @param translateKey
 * @param param
 * @returns {IValidationError}
 * @description createErrorObject function returns an object with the error message
 * for a field in the form in a proper format
 * @example createErrorObject('required', { param: 'email' }) => { translationKey: 'required', fallbackLanguageTranslation: 'This field is required', meta: { param: 'email' } }
 */
const createErrorObject = (translateKey, param) => ({
  translationKey: translateKey,
  fallbackLanguageTranslation: translateKey,
  meta: param || {}
});
function isFormGroup(toBeDetermined) {
  return !!toBeDetermined.children;
}
/**
 * @param stringifiedFunction
 * @returns {Function}
 * @description functionFromString returns a function from a stringified function
 * @example functionFromString('/Function((values) => {return values.medicalConditions.toString() === 'true'})/')
 * => (values) => values.medicalConditions.toString() === 'true'
 */
const functionFromString = stringifiedFunction => {
  try {
    const substr = stringifiedFunction.substring(10, stringifiedFunction.length - 2);
    return (0, eval)('(' + substr + ')');
  } catch (e) {
    console.log('Error parsing custom validation function in functionFromString. This issue wont cause an error, current validation was skipped', e);
    return () => false;
  }
};
/**
 * @param validationError - yup validation error
 * @param initialErrors - initial errors object to be merged with the yup validation errors
 * @returns {IValidationErrors} - returns an object with the errors for each field in the form
 * @description formatYupValidationErrors function returns an object with the errors for each field in the form
 * based on the yup validation error formatting to be used in the GenericForm component
 * @example formatYupValidationErrors(yupValidationError, { email: { property: 'email', constraints: { required: { translationKey: 'required', fallbackLanguageTranslation: 'This field is required', meta: {} } } } })
 */
const formatYupValidationErrors = (validationError, initialErrors = {}) => {
  const errors = _extends({}, initialErrors);
  for (const propertyValidationError of validationError.inner) {
    var _propertyValidationEr;
    let errorType = propertyValidationError.type;
    if (!errorType && (_propertyValidationEr = propertyValidationError.params) != null && _propertyValidationEr.regex) {
      var _propertyValidationEr2;
      if (!((_propertyValidationEr2 = propertyValidationError.params) != null && _propertyValidationEr2.regex)) {
        continue;
      }
      errorType = 'pattern';
    }
    if (!get(errors, `${propertyValidationError.path}`)) {
      errors[propertyValidationError.path] = {
        property: propertyValidationError.path,
        constraints: {}
      };
    }
    errors[propertyValidationError.path].constraints[errorType] = _extends({}, getFieldError(errorType, get(propertyValidationError, 'path', null), undefined, get(propertyValidationError.params, `${propertyValidationError.type}`)), {
      fallbackLanguageTranslation: propertyValidationError.message
    });
  }
  return errors;
};
/**
 * @param children
 * @param type - field type
 * @param acc - accumulator, used for recursion
 * @returns {string[]} - array of field names
 * @description getFieldNamesByType function returns an array of field names based on the field type
 * @example getFieldNamesByType(form, EFieldTypes.TEXT) => ['firstName', 'lastName']
 */
const getFieldNamesByType = (children, type, acc = []) => {
  return children.reduce((acc, child) => {
    if (isFormGroup(child)) {
      return getFieldNamesByType(child.children, type, acc);
    }
    if (child.type === type) {
      acc.push(child.fieldName);
    }
    return acc;
  }, acc);
};
const getAllFieldNames = (children, acc = []) => {
  return children.reduce((acc, child) => {
    if (isFormGroup(child)) {
      return getAllFieldNames(child.children, acc);
    }
    acc.push(child.fieldName);
    return acc;
  }, acc);
};
const flattenValues = obj => {
  const flattenedValues = _extends({}, obj);
  const keys = Object.keys(obj);
  keys.forEach(k => {
    if (typeof obj[k] === 'object') {
      flattenedValues[k] = obj[k].value;
    }
  });
  return flattenedValues;
};
/**
 * @param obj - object to trim
 * @returns {any} - object with trimmed values
 * @description trimValuesInObject function returns an object with trimmed values
 */
const trimValuesInObject = obj => {
  const trimmedValues = _extends({}, obj);
  const keys = Object.keys(obj);
  keys.forEach(k => obj[k] = typeof obj[k] == 'string' ? trimmedValues[k] = String(obj[k]).trim() : trimmedValues[k] = obj[k]);
  return trimmedValues;
};
const parseForm = form => {
  if (!(form != null && form.formatVersion)) {
    throw new Error('formatVersion is required');
  }
  switch (form.formatVersion) {
    case 1:
      return parseFormV1(form);
    case 2:
      return parseFormV2(form);
    default:
      throw new Error('The parsing function for this version of forms is not implemented');
  }
};

export { EFieldTypes, buildYupSchema, createErrorObject, formatYupValidationErrors, functionFromString, getAllFieldNames, getErrorMessages, getFieldError, getInitialValues, getRuleValueFromSchema, isFormGroup, makeCrossFieldValidation, parseForm, validateValues };
//# sourceMappingURL=index.modern.mjs.map
