import { isFormGroup } from './helpers';
import { IFormFieldV1, IFormGroupV1, IFormV1 } from './models';
import { EnumType, IFormFieldBase, IFormGroupBase } from './modelsCommon';
import { IFormSettings, IFormParsed } from './modelsParsed';

const buildOptionsFromChildren = <T extends EnumType>(
  children: (IFormFieldV1<T> | IFormGroupV1<T>)[],
  formOptions = {} as IFormSettings<T>,
): IFormSettings<T> => {
  return children.reduce((acc, child) => {
    if (isFormGroup(child)) {
      return buildOptionsFromChildren(child.children, acc);
    }

    const fieldLayout = child.grid || {
      md: 12,
      sm: 12,
      xs: 12,
    };

    acc[child.fieldName] = {
      crossFieldConditions: {
        renderCondition: child.renderCondition,
        disabledCondition: child.disabledCondition,
      },
      fieldLayout,
    };

    return acc;
  }, formOptions);
};

const sanitizeChildrenFunction = <T extends EnumType>(
  children: (IFormFieldV1<T> | IFormGroupV1<T>)[],
): (IFormFieldBase<T> | IFormGroupBase<T>)[] => {
  return children.map((child) => {
    if (isFormGroup(child)) {
      return {
        ...child,
        children: sanitizeChildrenFunction(child.children),
      } as IFormGroupBase<T>;
    }

    const { grid, renderCondition, disabledCondition, ...sanitizedChild } = child;
    return sanitizedChild;
  });
};

export const parseFormV1 = <T extends EnumType>(form: IFormV1<T>): IFormParsed<T> => {
  const {
    submitButtonText,
    cancelButtonText,
    validationObject: { crossFieldValidations, ...sanitizedValidationObject },
    children,
  } = form;

  const formOptions = buildOptionsFromChildren(children);

  const fieldsWithCrossFieldValidations = Object.keys(crossFieldValidations) as T[];

  fieldsWithCrossFieldValidations.forEach((key) => {
    formOptions[key].crossFieldValidations = crossFieldValidations[key];
  });

  const sanitizedChildren = sanitizeChildrenFunction(children);

  return {
    formatVersion: 'parsed',
    jsonSchema: sanitizedValidationObject,
    formSettings: formOptions,
    children: sanitizedChildren,
    submitButtonText,
    cancelButtonText,
  };
};
